@@ -1,16 +1,18 @@
 """CachedSession + BaseCache tests that use mocked responses only"""
 # TODO: This could be split up into some smaller test modules
 import json
+import pickle
 import time
 from collections import UserDict, defaultdict
 from datetime import datetime, timedelta
+from pathlib import Path
 from pickle import PickleError
 from unittest.mock import patch
 from urllib.parse import urlencode
 
 import pytest
 import requests
-from requests import Request
+from requests import Request, RequestException
 from requests.structures import CaseInsensitiveDict
 
 from requests_cache import ALL_METHODS, CachedResponse, CachedSession
@@ -35,6 +37,11 @@ def test_init_unregistered_backend():
         CachedSession(backend='nonexistent')
 
 
+def test_cache_path_expansion():
+    session = CachedSession('~', backend='filesystem')
+    assert session.cache.cache_dir == Path("~").expanduser()
+
+
 @patch.dict(BACKEND_CLASSES, {'mongo': get_placeholder_class()})
 def test_init_missing_backend_dependency():
     """Test that the correct error is thrown when a user does not have a dependency installed"""
@@ -72,6 +79,11 @@ def test_init_backend_class():
     assert session.cache.cache_name == 'test_cache'
 
 
+def test_pickle__disabled():
+    with pytest.raises(NotImplementedError):
+        pickle.dumps(CachedSession(backend='memory'))
+
+
 @pytest.mark.parametrize('method', ALL_METHODS)
 @pytest.mark.parametrize('field', ['params', 'data', 'json'])
 def test_all_methods(field, method, mock_session):
@@ -119,6 +131,12 @@ def test_all_methods__ignored_parameters__redacted(field, method, mock_session):
     assert 'access_token' not in cached_response.request.body.decode('utf-8')
 
 
+def test_params_positional_arg(mock_session):
+    mock_session.request('GET', MOCKED_URL, {'param_1': 1})
+    response = mock_session.request('GET', MOCKED_URL, {'param_1': 1})
+    assert 'param_1=1' in response.url
+
+
 def test_https(mock_session):
     assert mock_session.get(MOCKED_URL_HTTPS, verify=True).from_cache is False
     assert mock_session.get(MOCKED_URL_HTTPS, verify=True).from_cache is True
@@ -148,13 +166,8 @@ def test_response_history(mock_session):
 
 def test_repr(mock_session):
     """Test session and cache string representations"""
-    mock_session.expire_after = 10.5
-    mock_session.cache.responses['key'] = 'value'
-    mock_session.cache.redirects['key'] = 'value'
-    mock_session.cache.redirects['key_2'] = 'value'
-
-    assert mock_session.cache.cache_name in repr(mock_session) and '10.5' in repr(mock_session)
-    assert '2 redirects' in str(mock_session.cache) and '1 responses' in str(mock_session.cache)
+    assert repr(mock_session.cache) == '<SQLiteCache(name=http_cache)>'
+    assert str(mock_session.cache) == '<SQLiteCache(name=http_cache)>'
 
 
 def test_urls(mock_session):
@@ -509,9 +522,9 @@ def test_expired_request_error(mock_session):
     """Without stale_if_error (default), if there is an error while re-fetching an expired
     response, the request should be re-raised and the expired item deleted"""
     mock_session.stale_if_error = False
-    mock_session.expire_after = 0.01
+    mock_session.expire_after = 1
     mock_session.get(MOCKED_URL)
-    time.sleep(0.01)
+    time.sleep(1)
 
     with patch.object(mock_session.cache, 'save_response', side_effect=ValueError):
         with pytest.raises(ValueError):
@@ -522,12 +535,12 @@ def test_expired_request_error(mock_session):
 def test_stale_if_error__exception(mock_session):
     """With stale_if_error, expect to get old cache data if there is an exception during a request"""
     mock_session.stale_if_error = True
-    mock_session.expire_after = 0.2
+    mock_session.expire_after = 1
 
     assert mock_session.get(MOCKED_URL).from_cache is False
     assert mock_session.get(MOCKED_URL).from_cache is True
-    time.sleep(0.2)
-    with patch.object(mock_session.cache, 'save_response', side_effect=ValueError):
+    time.sleep(1)
+    with patch.object(mock_session.cache, 'save_response', side_effect=RequestException):
         response = mock_session.get(MOCKED_URL)
         assert response.from_cache is True and response.is_expired is True
 
@@ -535,12 +548,12 @@ def test_stale_if_error__exception(mock_session):
 def test_stale_if_error__error_code(mock_session):
     """With stale_if_error, expect to get old cache data if a response has an error status code"""
     mock_session.stale_if_error = True
-    mock_session.expire_after = 0.2
+    mock_session.expire_after = 1
     mock_session.allowable_codes = (200, 404)
 
     assert mock_session.get(MOCKED_URL_404).from_cache is False
 
-    time.sleep(0.2)
+    time.sleep(1)
     response = mock_session.get(MOCKED_URL_404)
     assert response.from_cache is True and response.is_expired is True
 
@@ -646,6 +659,21 @@ def test_remove_expired_responses(mock_session):
     assert len(mock_session.cache.responses) == 0
 
 
+def test_invalid_expiration(mock_session):
+    mock_session.settings.expire_after = 'tomorrow'
+    with pytest.raises(ValueError):
+        mock_session.get(MOCKED_URL)
+
+    mock_session.settings.expire_after = object()
+    with pytest.raises(TypeError):
+        mock_session.get(MOCKED_URL)
+
+    mock_session.settings.expire_after = None
+    mock_session.settings.urls_expire_after = {'*': 'tomorrow'}
+    with pytest.raises(ValueError):
+        mock_session.get(MOCKED_URL)
+
+
 def test_remove_expired_responses__error(mock_session):
     # Start with two cached responses, one of which will raise an error
     mock_session.get(MOCKED_URL)
@@ -712,8 +740,8 @@ def test_remove_expired_responses__per_request(mock_session):
     assert len(mock_session.cache.responses) == 1
 
 
-def test_per_request__expiration(mock_session):
-    """No per-session expiration is set, but then overridden with per-request expiration"""
+def test_per_request__enable_expiration(mock_session):
+    """No per-session expiration is set, but then overridden for a single request"""
     mock_session.expire_after = None
     response = mock_session.get(MOCKED_URL, expire_after=1)
     assert response.from_cache is False
@@ -724,6 +752,15 @@ def test_per_request__expiration(mock_session):
     assert response.from_cache is False
 
 
+def test_per_request__disable_expiration(mock_session):
+    """A per-session expiration is set, but then disabled for a single request"""
+    mock_session.expire_after = 60
+    response = mock_session.get(MOCKED_URL, expire_after=-1)
+    response = mock_session.get(MOCKED_URL, expire_after=-1)
+    assert response.from_cache is True
+    assert response.expires is None
+
+
 def test_per_request__prepared_request(mock_session):
     """The same should work for PreparedRequests with CachedSession.send()"""
     mock_session.expire_after = None
